# -*- coding: utf-8 -*-

"""
   wegene.APIHelper

   This file was automatically generated by APIMATIC BETA v2.0 on 02/22/2016
"""
import jsonpickle
import re

class APIHelper:

    """A Helper Class for various functions associated with API Calls.

    This class contains static methods for operations that need to be
    performed during API requests. All of the methods inside this class are
    static methods, there is no need to ever initialise an instance of this
    class.

    """

    @staticmethod
    def json_serialize(obj):
        """JSON Serialization of a given object.

        Args:
            obj (object): The object to serialise.

        Returns:
            str: The JSON serialized string of the object.

        """
        if obj is None:
            return None

        # Resolve any Names if it's one of our objects that needs to have this called on
        if isinstance(obj, list):
            value = list()

            for item in obj:
                try:
                    value.append(item.resolve_names())
                except (AttributeError, TypeError):
                    value.append(item)

            obj = value
        else:
            try:
                obj = obj.resolve_names()
            except (AttributeError, TypeError):
                obj = obj

        return jsonpickle.encode(obj, False)

    @staticmethod
    def json_deserialize(json):
        """JSON Deerialization of a given string.

        Args:
            json (str): The JSON serialized string to deserialize.

        Returns:
            dict: A dictionary representing the data contained in the
                JSON serialized string.

        """
        if json is None:
            return None

        return jsonpickle.decode(json)

    @staticmethod
    def append_url_with_template_parameters(url,
                                            parameters):
        """Replaces template parameters in the given url.

        Args:
            url (str): The query url string to replace the template parameters.
            parameters (dict): The parameters to replace in the url.

        Returns:
            str: Url with replaced parameters.

        """
        # Parameter validation
        if url is None:
            raise ValueError("url is null")
        if parameters is None:
            return url

        # Iterate and replace parameters
        for key in parameters:
            element = parameters[key]
            replace_value = ""

            # Load parameter value
            if element is None:
                replace_value = ""
            elif isinstance(element, list):
                replace_value = "/".join(element)
            else:
                replace_value = str(element)

            url = url.replace('{{{0}}}'.format(key),str(replace_value))

        return url

    @staticmethod
    def append_url_with_query_parameters(url,
                                         parameters):
        """Appends the given set of parameters to the given query string.

        Args:
            url (str): The query url string to append the parameters.
            parameters (dict): The parameters to append.

        Returns:
            str: Url with appended query parameters.

        """
        # Perform parameter validation
        if url is None:
            raise ValueError("url is null")
        if parameters is None:
            return url

        # Does the query string already have parameters?
        has_params = '?' in url

        # Iterate and replace parameters
        for key in parameters:
            element = parameters[key]

            # Ignore null values
            if element is None:
                continue

            # If already has parameters, use the &amp; to append new parameters
            separator = '&' if has_params else '?'

            if isinstance(element, list):
                url = url + '{0}{1}[]={2}'.format(separator, key, '&{0}[]='.format(key).join(element))
            else:
                url = url + '{0}{1}={2}'.format(separator, key, str(parameters[key]))

            # Indicate the url has params
            has_params = True

        return url

    @staticmethod
    def clean_url(url):
        """Validates and processes the given query Url to clean empty slashes.

        Args:
            url (str): The given query Url to process.

        Returns:
            str: Clean Url as string.

        """
        # Ensure that the urls are absolute
        regex = "^https?://[^/]+"
        match = re.match(regex, url)
        if match is None:
            raise ValueError('Invalid Url format.')

        # Remove redundant forward slashes
        protocol = match.group(0)
        query_url = url[len(protocol):]
        query_url = re.sub("//+", "/", query_url);

        return protocol + query_url
        return query_url

    @staticmethod
    def form_encode(obj,
                    instanceName):
        """Encodes a model in a form-encoded manner such as person[Name]

        Args:
            obj (object): The given Object to form encode.
            instanceName (string): The base name to appear before each entry
                for this object.

        Returns:
            dict: A dictionary of form encoded properties of the model.

        """
        # Resolve the names first
        value = APIHelper.resolve_name(obj)
        retval = dict()

        if value is None:
            return None

        # Loop through every item we need to send
        for item in value:
            if isinstance(value[item], list):
                # Loop through each item in the list and add it by number
                i = 0
                for entry in value[item]:
                    retval.update(APIHelper.form_encode(entry, instanceName + "[" + item + "][" + str(i) + "]"))
                    i += 1
            elif isinstance(value[item], dict):
                # Loop through each item in the dictionary and add it
                retval.update(APIHelper.form_encode(value[item], instanceName + "[" + item + "]"))
            else:
                # Add the current item
                retval[instanceName + "[" + item + "]"] = value[item]

        return retval

    @staticmethod
    def resolve_names(obj,
                      names,
                      retval):
        """Resolves parameters from their Model names to their API names.

        Args:
            obj (object): The given Object to resolve names for.
            names (dict): A dictionary containing a mapping from model name
                to API name.
            retval (dict): The dictionary to return which may or may not be
                empty (but must not be None).

        Returns:
            dict: A dictionary form of the model with properties in their API
                formats.

        """
        # Loop through all properties in this model
        for name in names:
            value = getattr(obj, name)

            if isinstance(value, list):
                # Loop through each item
                retval[names[name]] = list()
                for item in value:
                    retval[names[name]].append(APIHelper.resolve_name(item))
            elif isinstance(value, dict):
                # Loop through each item
                retval[names[name]] = dict()
                for key in value:
                    retval[names[name]][key] = APIHelper.resolve_name(value[key])
            else:
                retval[names[name]] = APIHelper.resolve_name(value)

        # Return the result
        return retval

    @staticmethod
    def resolve_name(value):
        """Resolves name for a given object

        If the object needs to be recursively resolved, this method will
        perform that recursive call.

        Args:
            value (object): A parameter to check if it needs to be recursively
                resolved.

        Returns:
            object: A resolved parameter which may either be a dict or a
                primative object.

        """
        # Check if the item also has to be resolved
        if value is not None and hasattr(value, "resolve_names") and callable(getattr(value, "resolve_names")):
            return value.resolve_names()
        else:
            # Not an object that needs resolving
            return value



# -*- coding: utf-8 -*-

"""
   wegene.APIException

   This file was automatically generated by APIMATIC BETA v2.0 on 02/22/2016
"""
class APIException(Exception):

    """Class that handles HTTP Exceptions when fetching API Endpoints.

    Attributes:
        reason (string): The reason (or error message) for the Exception to be
            raised.
        response_code (int): The HTTP Response Code from the API Request that
            caused this exception to be raised.
        response_body (string): The body that was retrieved during the API
            request.

    """

    def __init__(self,
                 reason,
                 response_code,
                 response_body):
        """Constructor for the APIException class

        Args:
            reason (string): The reason (or error message) for the Exception
                to be raised.
            response_code (int): The HTTP Response Code from the API Request
                that caused this exception to be raised.
            response_body (string): The body that was retrieved during the API
                request.

        """
        Exception.__init__(self, reason)
        self.response_code = response_code
        self.response_body = response_body



# -*- coding: utf-8 -*-

"""
   wegene.Controllers.HealthController

   This file was automatically generated by APIMATIC BETA v2.0 on 02/22/2016
"""
import unirest

from wegene.APIHelper import APIHelper
from wegene.Configuration import Configuration
from wegene.APIException import APIException


class HealthController(object):


    """A Controller to access Endpoints in the WeGeneAPILib API."""

    def get_risk(self,
                    profile_id,
                    report_id):
        """Does a POST request to /health/risk/{profile_id}.

        Health risk based on genetic information

        Args:
            profile_id (string): Genetic profile id
            report_id (string): Report Id for the specific health risk to
                look

        Returns:
            mixed: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        # The base uri for api requests
        query_builder = Configuration.BASE_URI

        # Prepare query string for API call
        query_builder += "/health/risk/{profile_id}"

        # Process optional template parameters
        query_builder = APIHelper.append_url_with_template_parameters(query_builder, {
            "profile_id": profile_id
        })

        # Validate and preprocess url
        query_url = APIHelper.clean_url(query_builder)

        # Prepare headers
        headers = {

            "Authorization": "Bearer "+ Configuration.o_auth_access_token,
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",

        }

        # Prepare parameters
        parameters = {
            "report_id": report_id
        }

        # Prepare and invoke the API call request to fetch the response
        response = unirest.post(query_url, headers=headers, params=parameters)

        # Error handling using HTTP status codes
        if response.code < 200 or response.code > 206:  # 200 = HTTP OK
            raise APIException("HTTP Response Not OK", response.code, response.body)

        return response.body

    def get_carrier(self,
                       profile_id,
                       report_id):
        """Does a POST request to /health/carrier/{profile_id}.

        Disease carrier information based on genetic information

        Args:
            profile_id (string): Genetic profile Id
            report_id (string): Report Id for the specific disease carrier to
                look

        Returns:
            mixed: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        # The base uri for api requests
        query_builder = Configuration.BASE_URI

        # Prepare query string for API call
        query_builder += "/health/carrier/{profile_id}"

        # Process optional template parameters
        query_builder = APIHelper.append_url_with_template_parameters(query_builder, {
            "profile_id": profile_id
        })

        # Validate and preprocess url
        query_url = APIHelper.clean_url(query_builder)

        # Prepare headers
        headers = {

            "Authorization": "Bearer "+ Configuration.o_auth_access_token,
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",

        }

        # Prepare parameters
        parameters = {
            "report_id": report_id
        }

        # Prepare and invoke the API call request to fetch the response
        response = unirest.post(query_url, headers=headers, params=parameters)

        # Error handling using HTTP status codes
        if response.code < 200 or response.code > 206:  # 200 = HTTP OK
            raise APIException("HTTP Response Not OK", response.code, response.body)

        return response.body

    def get_drug(self,
                    profile_id,
                    report_id):
        """Does a POST request to /health/drug/{profile_id}.

        Drug response based on genetic information

        Args:
            profile_id (string): Genetic profile Id
            report_id (string): Report Id for the specific drug response to
                look

        Returns:
            mixed: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """
        # The base uri for api requests
        query_builder = Configuration.BASE_URI

        # Prepare query string for API call
        query_builder += "/health/drug/{profile_id}"

        # Process optional template parameters
        query_builder = APIHelper.append_url_with_template_parameters(query_builder, {
            "profile_id": profile_id
        })

        # Validate and preprocess url
        query_url = APIHelper.clean_url(query_builder)

        # Prepare headers
        headers = {

            "Authorization": "Bearer "+ Configuration.o_auth_access_token,
            "user-agent": "APIMATIC 2.0",
            "accept": "application/json",

        }

        # Prepare parameters
        parameters = {
            "report_id": report_id
        }

        # Prepare and invoke the API call request to fetch the response
        response = unirest.post(query_url, headers=headers, params=parameters)

        # Error handling using HTTP status codes
        if response.code < 200 or response.code > 206:  # 200 = HTTP OK
            raise APIException("HTTP Response Not OK", response.code, response.body)

        return response.body



# -*- coding: utf-8 -*-

"""
   wegene.Models.GenotypeModel

   This file was automatically generated by APIMATIC BETA v2.0 on 02/22/2016
"""
from wegene.APIHelper import APIHelper

class GenotypeModel(object):

    """Implementation of the 'genotype' model.

    Genotype information

    Attributes:
        gene (string): Name of the gene
        genotype (string): Genotype of the user
        genotypeid (int): Genotype Id
        mag (string): Research value of the genotype
        orientation (string): Dominance or Recessive type
        repute (string): Beneficial/Harmful for the carrier
        rsid (string): RS Id
        snpid (int): SNP Id
        summary (string): Description of the genotype impact
        tsummary (string): Description of the genotype impact in plain Chinese
            texts

    """

    def __init__(self,
                 **kwargs):
        """Constructor for the GenotypeModel class

        Args:
            **kwargs: Keyword Arguments in order to initialise the
                object. Any of the attributes in this object are able to
                be set through the **kwargs of the constructor. The values
                that can be supplied and their types are as follows::

                    gene -- string -- Sets the attribute gene
                    genotype -- string -- Sets the attribute genotype
                    genotypeid -- int -- Sets the attribute genotypeid
                    mag -- string -- Sets the attribute mag
                    orientation -- string -- Sets the attribute orientation
                    repute -- string -- Sets the attribute repute
                    rsid -- string -- Sets the attribute rsid
                    snpid -- int -- Sets the attribute snpid
                    summary -- string -- Sets the attribute summary
                    tsummary -- string -- Sets the attribute tsummary

        """
        # Set all of the parameters to their default values
        self.gene = None
        self.genotype = None
        self.genotypeid = None
        self.mag = None
        self.orientation = None
        self.repute = None
        self.rsid = None
        self.snpid = None
        self.summary = None
        self.tsummary = None

        # Create a mapping from API property names to Model property names
        replace_names = {
            "gene": "gene",
            "genotype": "genotype",
            "genotypeid": "genotypeid",
            "mag": "mag",
            "orientation": "orientation",
            "repute": "repute",
            "rsid": "rsid",
            "snpid": "snpid",
            "summary": "summary",
            "tsummary": "tsummary",
        }

        # Parse all of the Key-Value arguments
        if kwargs is not None:
            for key in kwargs:
                # Only add arguments that are actually part of this object
                if key in replace_names:
                    setattr(self, replace_names[key], kwargs[key])

    def resolve_names(self):
        """Creates a dictionary representation of this object.

        This method converts an object to a dictionary that represents the
        format that the model should be in when passed into an API Request.
        Because of this, the generated dictionary may have different
        property names to that of the model itself.

        Returns:
            dict: The dictionary representing the object.

        """
        # Create a mapping from Model property names to API property names
        replace_names = {
            "gene": "gene",
            "genotype": "genotype",
            "genotypeid": "genotypeid",
            "mag": "mag",
            "orientation": "orientation",
            "repute": "repute",
            "rsid": "rsid",
            "snpid": "snpid",
            "summary": "summary",
            "tsummary": "tsummary",
        }

        retval = dict()

        return APIHelper.resolve_names(self, replace_names, retval)



# -*- coding: utf-8 -*-

"""
   wegene.Models.SportAdvice

   This file was automatically generated by APIMATIC BETA v2.0 on 02/22/2016
"""
from wegene.APIHelper import APIHelper

class SportAdvice(object):

    """Implementation of the 'sport_advice' model.

    Sport advice based on targeting losing weight of 500g/week

    Attributes:
        met (int): TODO: type description here.
        metabolic (int): TODO: type description here.
        sports_speed (int): Suggested speed if jogging
        sports_times (int): Suggested sport times
        sports_type (string): Suggested sport type
        total_consume (int): Suggested kcal to burn every week
        total_intake (int): Suggested kcal to intake every week

    """

    def __init__(self,
                 **kwargs):
        """Constructor for the SportAdvice class

        Args:
            **kwargs: Keyword Arguments in order to initialise the
                object. Any of the attributes in this object are able to
                be set through the **kwargs of the constructor. The values
                that can be supplied and their types are as follows::

                    met -- int -- Sets the attribute met
                    metabolic -- int -- Sets the attribute metabolic
                    sports_speed -- int -- Sets the attribute sports_speed
                    sports_times -- int -- Sets the attribute sports_times
                    sports_type -- string -- Sets the attribute sports_type
                    total_consume -- int -- Sets the attribute total_consume
                    total_intake -- int -- Sets the attribute total_intake

        """
        # Set all of the parameters to their default values
        self.met = None
        self.metabolic = None
        self.sports_speed = None
        self.sports_times = None
        self.sports_type = None
        self.total_consume = None
        self.total_intake = None

        # Create a mapping from API property names to Model property names
        replace_names = {
            "met": "met",
            "metabolic": "metabolic",
            "sports_speed": "sports_speed",
            "sports_times": "sports_times",
            "sports_type": "sports_type",
            "total_consume": "total_consume",
            "total_intake": "total_intake",
        }

        # Parse all of the Key-Value arguments
        if kwargs is not None:
            for key in kwargs:
                # Only add arguments that are actually part of this object
                if key in replace_names:
                    setattr(self, replace_names[key], kwargs[key])

    def resolve_names(self):
        """Creates a dictionary representation of this object.

        This method converts an object to a dictionary that represents the
        format that the model should be in when passed into an API Request.
        Because of this, the generated dictionary may have different
        property names to that of the model itself.

        Returns:
            dict: The dictionary representing the object.

        """
        # Create a mapping from Model property names to API property names
        replace_names = {
            "met": "met",
            "metabolic": "metabolic",
            "sports_speed": "sports_speed",
            "sports_times": "sports_times",
            "sports_type": "sports_type",
            "total_consume": "total_consume",
            "total_intake": "total_intake",
        }

        retval = dict()

        return APIHelper.resolve_names(self, replace_names, retval)
